-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

newTalent{
	name = "Air Strike",
	type = {"technique/arcane-air", 1},
	require = techs_req1,
	points = 5,
	random_ego = "attack",
	cooldown = 8,
	stamina = 12,
	tactical = { ATTACK = { weapon = 2 }, DISABLE = { silence = 2 } },
	getDuration = function(self,t) return math.floor(3+math.floor(self:getTalentLevel(t))) end,
	action = function(self, t)
		--figure out which weapon (or none) to use
		local weapon = nil
		local offweapon = nil
		if self:getInven("MAINHAND") then
			weapon = self:getInven("MAINHAND")[1]
			if not weapon or not weapon.combat then
				weapon = nil
			end
		end
		if self:getInven("OFFHAND") then
			offweapon = self:getInven("OFFHAND")[1]
			if not offweapon or not offweapon.combat then
				offweapon = nil
			end
			if weapon == nil then
				weapon = offweapon
				offweapon = nil
			end
		end
		if self:attr("disarmed") or not weapon then
			weapon = self.combat
			offweapon = nil
		end

		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if target then
			if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end
			local speed, hit = self:attackTargetWith(target, weapon, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			-- Try to silence !
			if hit then
				if target:canBe("silence") then
					target:setEffect(target.EFF_SILENCED, t.getDuration(self,t), {apply_power=self:combatPhysicalpower()})
				else
					game.logSeen(target, "%s resists!", target.name:capitalize())
				end
			end
			--If dual wielding, attack with second weapon
			if offweapon then
				local speed, hit = self:attackTargetWith(target, offweapon, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			end
		end
		--Even if we miss or no target, the power must explode in a ball!
		if self:getTalentLevel(t) >= 4 then
			local dur = t.getDuration(self,t)
			local chance = math.floor(self:getTalentLevel(t) * 5)
			self:project({type="ball", radius=math.floor(self:getTalentLevel(t)-3), selffire=false, friendlyfire=false}, x, y, DamageType.AIR_STRIKE_BALL, {dur=dur,chance=chance})
		end
		if not self:isTalentActive(self.T_AIR_STANCE) then self:forceUseTalent(self.T_AIR_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true}) end
		return true
	end,
	--Learn and unlearn effects, add/remove air stance
	--Increase/Decrease spell/air mastery by 0.03
	on_learn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 1 then
			self:learnTalent(self.T_AIR_STANCE, true, nil, {no_unlearn=true})
		end
		self:setTalentTypeMastery("spell/air", self:getTalentTypeMastery("spell/air") + 0.03)
	end,
	on_unlearn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 0 then
			self:unlearnTalent(self.T_AIR_STANCE)
		end
		self:setTalentTypeMastery("spell/air", self:getTalentTypeMastery("spell/air") - 0.03)
	end,

	info = function(self, t)
		local radius = math.max(math.floor(self:getTalentLevel(t)-3),0)
		local chance = math.floor(self:getTalentLevel(t) * 5)
		local dur = t.getDuration(self,t)
		return ([[Strike with the power of Air. Silences target for %d turns and deals %d%% damage.  Chance to silence increases with Physical Power.
		At level 4 it blasts a ball of air of radius %d which has a %d%% chance of silencing enemies as well.  After use, you change to the Air Stance.]]):
		format(math.floor(3+self:getTalentLevel(t)),100 * self:combatTalentWeaponDamage(t, 1, 1.5),radius,chance)
	end,
}

newTalent{
	name = "Air Stance",
	type = {"technique/elemental-stance", 1},
	points = 1,
	mode = "sustained",
	cooldown = 0,
	no_energy = true,
	elemental_stance = true,
	getStanceAccuracyMod = function(self,t) return math.ceil((self:combatSpellpower() / 10.0) * (1 +self:getTalentLevel(self.T_ARCANE_STANCE))) end,
	getStanceLightningMod = function(self,t) return (self:combatSpellpower() / 10.0) * (1 + self:getTalentLevel(self.T_ARCANE_STANCE)) end,
	on_pre_use = function(self, t, silent)
		-- prevent anyone from activating other than without the "force_talent_ignore_ressources" flag
		if not self:attr("force_talent_ignore_ressources") then
			if not silent then game.logPlayer(self, "You can not active Air Stance directly.  Use Air Strike to activate it.") end
			return false
		end
		return true
	end,
	activate = function(self,t)
		local stance_turned_off = nil
		self:attr("air_stance", 1)
		if self:isTalentActive(self.T_WATER_STANCE) then
			self:forceUseTalent(self.T_WATER_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:isTalentActive(self.T_EARTH_STANCE) then
			self:forceUseTalent(self.T_EARTH_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:isTalentActive(self.T_FIRE_STANCE) then
			self:forceUseTalent(self.T_FIRE_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if not stance_turned_off and self:isTalentActive(self.T_ARCANE_DESTRUCTION) then
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		return {
			atk = self:addTemporaryValue("combat_atk", t.getStanceAccuracyMod(self,t)),
			dam = self:addTemporaryValue("melee_project", {[DamageType.LIGHTNING] = t.getStanceLightningMod(self, t)}),
		}
	end,
	deactivate = function(self,t,p)
		self:attr("air_stance",-1)
		--turn Arcane Destruction on and off if on.
		if self:isTalentActive(self.T_ARCANE_DESTRUCTION) then
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		self:removeTemporaryValue("combat_atk", p.atk)
		self:removeTemporaryValue("melee_project", p.dam)
		return true
	end,
	info = function(self, t)
		local atk = t.getStanceAccuracyMod(self,t)
		local dam = t.getStanceLightningMod(self,t)
		return ([[Enter the elemental stance of Air by using the Air Strike talent.  Air Stance increases your accuracy by %d and deals %0.2f lightning damage every time you hit with a weapon.
		Benefits increase with the Arcane Stance talent.]])
		:format(atk, dam)
	end,
}
