-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

newTalent{
	name = "Water Strike",
	type = {"technique/arcane-water", 1},
	require = techs_req1,
	points = 5,
	random_ego = "attack",
	cooldown = 8,
	stamina = 12,
	tactical = { ATTACK = { weapon = 2 }, DISABLE = { stun = 2 } },
	action = function(self, t)
		--figure out which weapon (or none) to use
		local weapon = nil
		local offweapon = nil
		if self:getInven("MAINHAND") then
			weapon = self:getInven("MAINHAND")[1]
			if not weapon or not weapon.combat then
				weapon = nil
			end
		end
		if self:getInven("OFFHAND") then
			offweapon = self:getInven("OFFHAND")[1]
			if not offweapon or not offweapon.combat then
				offweapon = nil
			end
			if weapon == nil then
				weapon = offweapon
				offweapon = nil
			end
		end
		if self:attr("disarmed") or not weapon then
			weapon = self.combat
			offweapon = nil
		end

		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if target then
			if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end

			--If dual wielding, attack with second weapon first, so as to not affect the ice block
			if offweapon then
				local speed, hit = self:attackTargetWith(target, offweapon, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			end

			local speed, hit = self:attackTargetWith(target, weapon, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			-- Try to stun !
			if hit then
				if target:canBe("stun") then
					target:setEffect(target.EFF_FROZEN, 2, {hp=70 + self:combatPhysicalpower() * self:getTalentLevel(t), apply_power=self:combatPhysicalpower(), min_dur=1})
					game.flyers:add(x, y, 30, (rng.range(0,2)-1) * 0.5, -3, "Frozen!", {0,255,155})
				else
					game.flyers:add(x, y, 30, (rng.range(0,2)-1) * 0.5, -3, "Resist!", {0,255,155})
					game.logSeen(target, "%s resists!", target.name:capitalize())
				end
			end
		end
		--Even if we miss or no target, the power must explode in a ball!
		if self:getTalentLevel(t) >= 4 then
			local dur = 2
			local chance = math.floor(self:getTalentLevel(t) * 5)
			local hp = 70 + self:combatPhysicalpower() * self:getTalentLevel(t)
			self:project({type="ball", radius=math.floor(self:getTalentLevel(t)-3), selffire=false, friendlyfire=false}, x, y, DamageType.WATER_STRIKE_BALL, {dur=dur,chance=chance,hp=hp})
		end
		if not self:isTalentActive(self.T_WATER_STANCE) then self:forceUseTalent(self.T_WATER_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true}) end
		return true
	end,
	--Learn and unlearn effects, add/remove water stance
	--Increase/Decrease spell/water mastery by 0.03
	on_learn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 1 then
			self:learnTalent(self.T_WATER_STANCE, true, nil, {no_unlearn=true})
		end
		self:setTalentTypeMastery("spell/water", self:getTalentTypeMastery("spell/water") + 0.03)
	end,
	on_unlearn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 0 then
			self:unlearnTalent(self.T_WATER_STANCE)
		end
		self:setTalentTypeMastery("spell/water", self:getTalentTypeMastery("spell/water") - 0.03)
	end,

	info = function(self, t)
		local radius = math.max(math.floor(self:getTalentLevel(t)-3),0)
		local chance = math.floor(self:getTalentLevel(t) * 5)
		return ([[Strike with the power of Water. Freezes target for 2 turns and deals %d%% damage.  Chance to freeze increases with Physical Power.
		At level 4 it blasts a ball of water of radius %d which has a %d%% chance of freezing enemies as well.  After use, you change to the Water Stance.]]):
		format(100 * self:combatTalentWeaponDamage(t, 1, 1.5),radius,chance)
	end,
}

newTalent{
	name = "Water Stance",
	type = {"technique/elemental-stance", 1},
	points = 1,
	mode = "sustained",
	cooldown = 0,
	no_energy = true,
	elemental_stance = true,
	getStanceHealMod = function(self,t) return (self:combatSpellpower() / 1000.0) * (1 +self:getTalentLevel(self.T_ARCANE_STANCE)) end,
	getStanceRegenMod = function(self,t) return (self:combatSpellpower() / 100.0) * (1 + self:getTalentLevel(self.T_ARCANE_STANCE)) end,
	on_pre_use = function(self, t, silent)
		-- prevent anyone from activating other than without the "force_talent_ignore_ressources" flag
		if not self:attr("force_talent_ignore_ressources") then
			if not silent then game.logPlayer(self, "You can not active Water Stance directly.  Use Water Strike to activate it.") end
			return false
		end
		return true
	end,
	activate = function(self,t)
		local stance_turned_off = nil
		self:attr("water_stance", 1)
		if self:isTalentActive(self.T_AIR_STANCE) then
			self:forceUseTalent(self.T_AIR_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:isTalentActive(self.T_EARTH_STANCE) then
			self:forceUseTalent(self.T_EARTH_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:isTalentActive(self.T_FIRE_STANCE) then
			self:forceUseTalent(self.T_FIRE_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if not stance_turned_off and self:isTalentActive(self.T_ARCANE_DESTRUCTION) then
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		return {
			heal_inc = self:addTemporaryValue("healing_factor", t.getStanceHealMod(self,t)),
			regen_inc = self:addTemporaryValue("life_regen", t.getStanceRegenMod(self,t)),
		}
	end,
	deactivate = function(self,t,p)
		self:attr("water_stance", -1)
		--turn Arcane Destruction on and off if on.
		if self:isTalentActive(self.T_ARCANE_DESTRUCTION) then
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		self:removeTemporaryValue("healing_factor", p.heal_inc)
		self:removeTemporaryValue("life_regen", p.regen_inc)
		return true
	end,
	info = function(self, t)
		local healinc = t.getStanceHealMod(self, t)
		local regeninc = t.getStanceRegenMod(self, t)
		return ([[Enter the elemental stance of Water by using the Water Strike talent.  Water Stance increases your healing effects by %d%% and your regeneration by %0.2f.  Benefits increase with the Arcane Stance talent.]])
		:format(healinc * 100, regeninc)
	end,
}
